#! /usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("dotenv").config();
const os = require("os");
const fs = require("fs");
const commander_1 = require("commander");
const anchor = __importStar(require("@project-serum/anchor"));
const nodewallet_1 = __importDefault(require("@project-serum/anchor/dist/cjs/nodewallet"));
const web3_js_1 = require("@solana/web3.js");
const index_1 = require("./index");
const constants = __importStar(require("./constants"));
const __path = process.cwd();
const figlet = require("figlet");
const program = new commander_1.Command();
const MAINNET_RPC = "https://api-mainnet-beta.renec.foundation:8899/";
const TESTNET_RPC = "https://api-testnet.renec.foundation:8899/";
const LOCAL_RPC = "http://localhost:8899/";
const TOKENS = {
    REUSD: constants.REUSD,
    REVND: constants.REVND,
    REBTC: constants.REBTC,
    REETH: constants.REETH,
    RENEC: constants.RENEC,
    RENGN: constants.RENGN,
};
const TESTNET_TOKENS = {
    REUSD: constants.REUSD_TESTNET,
    REVND: constants.REVND_TESTNET,
    REBTC: constants.REBTC_TESTNET,
    REETH: constants.REETH_TESTNET,
    RENEC: constants.RENEC_TESTNET,
};
console.log(figlet.textSync("Oracle Program"));
console.log("");
const __homePath = os.homedir();
const defaultOwnerPath = __homePath + "/.config/solana/id.json";
function connectCluster(rpc, ownerKey, programId) {
    return __awaiter(this, void 0, void 0, function* () {
        const connection = new web3_js_1.Connection(rpc);
        const wallet = new nodewallet_1.default((0, index_1.keypairFromJson)(ownerKey));
        const anchorProvider = new anchor.Provider(connection, wallet, anchor.Provider.defaultOptions());
        // const peggedProgram: any = new Program(ProgramIDL as Idl, programId, anchorProvider);
        // const program = peggedProgram as Program<Oracle>;
        // return program;
        const ctx = index_1.Context.withProvider(anchorProvider, programId);
        return ctx;
    });
}
function getContext(network, ownerPath = __path + "/sdk/src/.wallets/owner.json") {
    return __awaiter(this, void 0, void 0, function* () {
        const validNetworks = ["mainnet", "testnet", "localnet"];
        if (!validNetworks.includes(network)) {
            console.log("Error: Invalid network");
            throw new Error("Invalid network");
        }
        let rpc = MAINNET_RPC;
        let programId = constants.ORACLE_PROGRAM_ID;
        if (network === "testnet") {
            rpc = TESTNET_RPC;
            programId = constants.ORACLE_PROGRAM_ID_TESTNET;
        }
        if (network === "localnet") {
            rpc = LOCAL_RPC;
            programId = constants.ORACLE_PROGRAM_ID_TESTNET;
        }
        const ownerKey = JSON.parse(fs.readFileSync(ownerPath));
        const ctx = yield connectCluster(rpc, ownerKey, programId);
        const { provider } = ctx;
        const { wallet } = provider;
        console.log(`Connected to ${network} cluster`);
        console.log(`RPC: ${rpc}`);
        console.log(`Owner: ${wallet.publicKey.toBase58()}`);
        console.log(`Program Id: ${programId.toBase58()}`);
        console.log();
        return ctx;
    });
}
function getPair(ctx, quote, base, version, network = "mainnet") {
    return __awaiter(this, void 0, void 0, function* () {
        if (!base || base === "" || !quote || quote === "") {
            throw new Error("Error: --baseToken and --quoteToken is required");
        }
        if (isNaN(version)) {
            throw new Error("Error: -v or --version must be valid number");
        }
        const _version = parseInt(version);
        let quoteToken;
        let baseToken;
        try {
            try {
                quoteToken = new web3_js_1.PublicKey(quote);
            }
            catch (error) {
                quoteToken =
                    network === "mainnet"
                        ? TOKENS[quote.toUpperCase()]
                        : TESTNET_TOKENS[quote.toUpperCase()];
            }
            try {
                baseToken = new web3_js_1.PublicKey(base);
            }
            catch (error) {
                baseToken =
                    network === "mainnet"
                        ? TOKENS[base.toUpperCase()]
                        : TESTNET_TOKENS[base.toUpperCase()];
            }
            console.log();
            console.log("quoteToken", quoteToken.toBase58());
            console.log("baseToken", baseToken.toBase58());
            console.log();
        }
        catch (error) {
            throw new Error("Cannot parse mintToken or lockToken");
        }
        const productClient = yield index_1.ProductClient.getProduct(ctx, quoteToken, baseToken, _version);
        return productClient;
    });
}
function validateNumber(number, name) {
    if (isNaN(number)) {
        throw new Error(`Error: ${name} must be valid number`);
    }
}
program
    .command("init")
    .description("initialize a new oracle controller.")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network } = params;
    console.log("Initializing a new oracle controller.");
    console.log("params:", params);
    if (isNaN(version)) {
        console.log("Error: -v or --version must be valid number");
        return;
    }
    version = parseInt(version);
    const ctx = yield getContext(network);
    const program = ctx.program;
    const pda = new index_1.PDA(program.programId);
    const controller = pda.controller(version);
    const signature = yield program.rpc.initialize(controller.bump, version, {
        accounts: {
            authority: program.provider.wallet.publicKey,
            controller: controller.key,
            systemProgram: anchor.web3.SystemProgram.programId,
            rent: web3_js_1.SYSVAR_RENT_PUBKEY,
        },
    });
    console.log(`oracle controller: ${controller.key.toBase58()}`);
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
//yarn cli addPair -n localnet -qs VND -qt FitRv8WbYZVfRZSc2nU2HjwBJv4f69JjPtcmo3GL7amh -bs USD -bt 14qVe8V34bXt7prLKRrkP5spScRQRJeoosojdrZDKm31 -max 25000.23 -min 22000.32
program
    .command("addPair")
    .description("add a new oracle pair.")
    .option("-qs, --quoteSymbol <string>", "The symbol of quote. Example: VND")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bs, --baseSymbol <string>", "The symbol of base. Example: USD")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-max, --maxPrice <number>", "The maximum price. Example: maximum 1 reUSD = 25000.83 reVND")
    .option("-min, --minPrice <number>", "The minimum price. Example: minimum 1 reUSD = 22000.23 reVND")
    .option("-d, --decimals <number>", "The decimals of price. Value in range [0 -> 10)")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteSymbol, quoteToken, baseSymbol, baseToken, maxPrice, minPrice, decimals, } = params;
    console.log("Adding a new oracle pair.");
    console.log("params:", params);
    if (!quoteSymbol || quoteSymbol === "") {
        console.log("Error: -qs, --quoteSymbol is required");
        return;
    }
    if (!baseSymbol || baseSymbol === "") {
        console.log("Error: -bs, --baseSymbol is required");
        return;
    }
    if (!quoteToken || quoteToken === "") {
        console.log("Error: -qt, --quoteToken is required");
        return;
    }
    if (!baseToken || baseToken === "") {
        console.log("Error: -bt, --baseToken is required");
        return;
    }
    quoteToken = new web3_js_1.PublicKey(quoteToken);
    baseToken = new web3_js_1.PublicKey(baseToken);
    if (isNaN(maxPrice)) {
        console.log("Error: -max, --maxPrice must be valid number");
        return;
    }
    if (isNaN(minPrice)) {
        console.log("Error: -min, --minPrice must be valid number");
        return;
    }
    if (isNaN(decimals)) {
        console.log("Error: -d, --decimals must be valid number");
        return;
    }
    if (decimals < 0 || decimals > 10) {
        console.log("Error: -d, --decimals must be in [0:10] range");
        return;
    }
    if (isNaN(version)) {
        console.log("Error: -v or --version must be valid number");
        return;
    }
    version = parseInt(version);
    const ctx = yield getContext(network);
    const program = ctx.program;
    const { wallet } = program.provider;
    const expo = -1 * decimals;
    const windowSize = 20; // 20s
    const acceptableChangePercent = 500; // 5%
    const tx = yield index_1.ProductClient.new(ctx, index_1.AssetType.Forex, baseSymbol, baseToken, quoteSymbol, quoteToken, expo, maxPrice, minPrice, windowSize, acceptableChangePercent, wallet.publicKey, version);
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("checkRelayer")
    .description(".")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-r, --relayer <string>", "The relayer address")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, relayer } = params;
    console.log("Adding a new oracle relayer.");
    console.log("params:", params);
    if (!relayer || relayer === "") {
        console.log("Error: -r, --relayer is required");
        return;
    }
    if (!quoteToken || quoteToken === "") {
        console.log("Error: -qt, --quoteToken is required");
        return;
    }
    if (!baseToken || baseToken === "") {
        console.log("Error: -bt, --baseToken is required");
        return;
    }
    quoteToken = new web3_js_1.PublicKey(quoteToken);
    baseToken = new web3_js_1.PublicKey(baseToken);
    relayer = new web3_js_1.PublicKey(relayer);
    if (isNaN(version)) {
        console.log("Error: -v or --version must be valid number");
        return;
    }
    version = parseInt(version);
    const ctx = yield getContext(network);
    const productClient = yield index_1.ProductClient.getProduct(ctx, quoteToken, baseToken, version);
    const _relayer = yield productClient.getPublisher(relayer);
    // const tx = await productClient.addPublisher(productClient.ctx.wallet.publicKey, relayer);
    // const signature = await tx.buildAndExecute();
    // console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE", _relayer);
}));
program
    .command("addRelayer")
    .description("add a new oracle relayer.")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-r, --relayer <string>", "The relayer address")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .option("--ownerPath <string>", "Path to owner.json", defaultOwnerPath)
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, relayer, ownerPath } = params;
    console.log("Adding a new oracle relayer.");
    console.log("params:", params);
    if (!relayer || relayer === "") {
        console.log("Error: -r, --relayer is required");
        return;
    }
    relayer = new web3_js_1.PublicKey(relayer);
    const ctx = yield getContext(network, ownerPath);
    const productClient = yield getPair(ctx, quoteToken, baseToken, version, network);
    const tx = yield productClient.addPublisher(productClient.ctx.wallet.publicKey, relayer);
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("rmRelayer")
    .description("remove the oracle relayer.")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-r, --relayer <string>", "The relayer address")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, relayer } = params;
    console.log("Removing the oracle relayer.");
    console.log("params:", params);
    if (!relayer || relayer === "") {
        console.log("Error: -r, --relayer is required");
        return;
    }
    if (!quoteToken || quoteToken === "") {
        console.log("Error: -qt, --quoteToken is required");
        return;
    }
    if (!baseToken || baseToken === "") {
        console.log("Error: -bt, --baseToken is required");
        return;
    }
    quoteToken = new web3_js_1.PublicKey(quoteToken);
    baseToken = new web3_js_1.PublicKey(baseToken);
    relayer = new web3_js_1.PublicKey(relayer);
    if (isNaN(version)) {
        console.log("Error: -v or --version must be valid number");
        return;
    }
    version = parseInt(version);
    const ctx = yield getContext(network);
    const productClient = yield index_1.ProductClient.getProduct(ctx, quoteToken, baseToken, version);
    const tx = yield productClient.rmPublisher(productClient.ctx.wallet.publicKey, relayer);
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("setSafeRange")
    .description("remove the oracle relayer.")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-max, --maxPrice <number>", "The maximum price. Example: maximum 1 reUSD = 25000.83 reVND")
    .option("-min, --minPrice <number>", "The minimum price. Example: minimum 1 reUSD = 22000.23 reVND")
    .option("-d, --decimals <number>", "The decimals of price. Value in range [0 -> 10)")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, decimals, minPrice, maxPrice } = params;
    console.log("Removing the oracle relayer.");
    console.log("params:", params);
    if (!quoteToken || quoteToken === "") {
        console.log("Error: -qt, --quoteToken is required");
        return;
    }
    if (!baseToken || baseToken === "") {
        console.log("Error: -bt, --baseToken is required");
        return;
    }
    quoteToken = new web3_js_1.PublicKey(quoteToken);
    baseToken = new web3_js_1.PublicKey(baseToken);
    if (isNaN(maxPrice)) {
        console.log("Error: -max, --maxPrice must be valid number");
        return;
    }
    if (isNaN(minPrice)) {
        console.log("Error: -min, --minPrice must be valid number");
        return;
    }
    if (isNaN(decimals)) {
        console.log("Error: -d, --decimals must be valid number");
        return;
    }
    if (decimals < 0 || decimals > 10) {
        console.log("Error: -d, --decimals must be in [0:10] range");
        return;
    }
    if (isNaN(version)) {
        console.log("Error: -v or --version must be valid number");
        return;
    }
    version = parseInt(version);
    const ctx = yield getContext(network);
    const productClient = yield index_1.ProductClient.getProduct(ctx, quoteToken, baseToken, version);
    const tx = yield productClient.setSafeRange(productClient.ctx.wallet.publicKey, maxPrice, minPrice, decimals * -1);
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("setDynamicLimit")
    .description("Set dynamic limit.")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-ws, --windowSize <number>", "The window size time. Example: 86400s.")
    .option("-p, --acceptalePercent <number>", "Acceptale change percent . Example: 1 % = 100")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .option("--ownerPath <string>", "Path to owner.json", defaultOwnerPath)
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, windowSize, acceptalePercent, ownerPath } = params;
    validateNumber(windowSize, "windowSize");
    validateNumber(acceptalePercent, "acceptalePercent");
    const ctx = yield getContext(network, ownerPath);
    const pair = yield getPair(ctx, quoteToken, baseToken, version, network);
    const tx = yield pair.setDynamicLimit(ctx.wallet.publicKey, windowSize, acceptalePercent);
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("postPrice")
    .description("Relayer posts a new price.")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-p, --price <number>", "New price")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .option("--ownerPath <string>", "Path to owner.json", defaultOwnerPath)
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, price, ownerPath } = params;
    validateNumber(price, "price");
    const ctx = yield getContext(network, ownerPath);
    const pair = yield getPair(ctx, quoteToken, baseToken, version, network);
    const tx = yield pair.postPrice(price, ctx.wallet.publicKey);
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("getPriceOfPair")
    .description("Get price of a pair information.")
    .option("-qt, --quoteToken <string>", "The token of quote. Example: reVND")
    .option("-bt, --baseToken <string>", "The token of base. Example: reUSD")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-v, --version <number>", "Version of the controller", "1")
    .option("--ownerPath <string>", "Path to owner.json", defaultOwnerPath)
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { version, network, quoteToken, baseToken, ownerPath } = params;
    console.log("Removing the oracle relayer.");
    console.log("params:", params);
    const ctx = yield getContext(network, ownerPath);
    const productClient = yield getPair(ctx, quoteToken, baseToken, version, network);
    const price = yield productClient.getPrice();
    const priceMemo = yield productClient.priceMemoData;
    console.log("priceMemo", priceMemo);
    const expo = productClient.productData.expo;
    let maxPrice = productClient.productData.maxPrice.toNumber();
    let minPrice = productClient.productData.minPrice.toNumber();
    console.log("expo:", expo);
    if (expo < 0) {
        maxPrice = maxPrice / Math.pow(10, Math.abs(expo));
        minPrice = minPrice / Math.pow(10, Math.abs(expo));
    }
    if (expo > 0) {
        maxPrice = maxPrice * Math.pow(10, expo);
        minPrice = minPrice * Math.pow(10, expo);
    }
    console.log("maxPrice:", maxPrice);
    console.log("minPrice:", minPrice);
    console.log(price);
    console.log("productKey", productClient.productKey.toBase58());
    console.log("priceKey", productClient.productData.priceAccount.toBase58());
    console.log("DONE");
}));
program.parse();
