/// <reference types="bn.js" />
import { BN } from "@project-serum/anchor";
import { PublicKey } from "@solana/web3.js";
import { TransactionBuilder } from "@orca-so/common-sdk";
import { Context, PDA } from "..";
import { ProductData, PriceData, PriceResult, PublisherData, PriceMemoData } from "../types";
export declare class ProductClient {
    ctx: Context;
    productData: ProductData;
    priceData: PriceData;
    productKey: PublicKey;
    priceMemoData: PriceMemoData | null;
    pda: PDA;
    constructor(ctx: Context, productKey: PublicKey, productData: ProductData, priceData: PriceData, priceMemoData: PriceMemoData | null, pda: PDA);
    static getProduct(ctx: Context, quote: PublicKey, base: PublicKey, version?: number): Promise<ProductClient>;
    static new(ctx: Context, assetType: object, baseCurrency: string, baseMint: PublicKey, quoteCurrency: string, quoteMint: PublicKey, expo: number, maxPrice: number, minPrice: number, windowSize: number, acceptableChangePercent: number, authority: PublicKey, version?: number): Promise<TransactionBuilder>;
    addPublisher(authority: PublicKey, authorityPublisher: PublicKey): Promise<TransactionBuilder>;
    rmPublisher(authority: PublicKey, authorityPublisher: PublicKey): Promise<TransactionBuilder>;
    setSafeRange(authority: PublicKey, maxPrice: number, minPrice: number, expo: number | undefined): Promise<TransactionBuilder>;
    setDynamicLimit(authority: PublicKey, windowSize: number, acceptalePercent: number): Promise<TransactionBuilder>;
    postPrice(newPrice: number, authorityPublisher: PublicKey): Promise<TransactionBuilder>;
    getPublisher(authorityPublisher: PublicKey): Promise<PublisherData>;
    getPrice(refresh?: boolean): Promise<PriceResult>;
    static convertToPriceFormat(number: number, desiredExpo: number): {
        price: BN;
        expo: number;
    };
    refresh(): Promise<ProductClient>;
}
